"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const cli_ux_1 = require("cli-ux");
const nodegit_1 = require("nodegit");
const base_command_1 = require("../base-command");
const github_client_1 = require("../github-client");
const jira_client_1 = require("../jira-client");
const prompt_validators_1 = require("../lib/prompt-validators");
class Pr extends base_command_1.default {
    async run() {
        const { flags } = this.parse(Pr);
        const config = await this.getConfig();
        // Jira
        if (flags.interactive) {
            if (!flags['ticket-id']) {
                flags['ticket-id'] = await this.promptInput({ message: 'Jira ticket ID', validator: prompt_validators_1.Validators.required });
            }
        }
        const jiraTicketId = flags['ticket-id'] || this.error('Missing ticket ID!');
        const jiraTicket = await this.getJiraTicket({ accessToken: config.jiraAccessToken, host: config.jiraHost, username: config.jiraEmail, ticketId: jiraTicketId });
        const jiraTicketURL = `https://${config.jiraHost}/browse/${jiraTicket.key}`;
        // Github
        if (flags.interactive) {
            if (!flags['base-branch']) {
                const choices = await this.getGitBranches();
                flags['base-branch'] = await this.promptChoices({ message: 'Base branch', choices });
            }
            if (!flags['pr-title']) {
                flags['pr-title'] = await this.promptInput({ message: 'Custom pull request title', default: jiraTicket.fields.summary });
            }
            if (!flags.description) {
                flags.description = await this.promptInput({ message: 'Pull request description' });
            }
            if (!flags.draft) {
                flags.draft = await this.promptConfirm({ message: 'Draft pull request?' });
            }
        }
        const baseBranch = flags['base-branch'] || this.error('Missing base branch!');
        const prTitle = `[${jiraTicket.key}] ${flags['pr-title'] || jiraTicket.fields.summary}`;
        const prDescription = flags.description ? `${flags.description}\n${jiraTicketURL}` : jiraTicketURL;
        const draft = flags.draft;
        try {
            const pr = await this.makePullRequest({ accessToken: config.githubAccessToken, base: baseBranch, title: prTitle, description: prDescription, draft });
            this.log(pr.html_url);
        }
        catch (error) {
            this.log(error.response.data);
            this.error(error.message);
        }
    }
    async getGitBranches() {
        const repo = await nodegit_1.Repository.open('.');
        const references = await repo.getReferences();
        return references
            .filter(ref => ref.isBranch())
            .map(ref => ref.shorthand());
    }
    async getJiraTicket(args) {
        let jiraClient = new jira_client_1.JiraClient({
            username: args.username,
            accessToken: args.accessToken,
            host: args.host
        });
        cli_ux_1.default.action.start('Fetching Jira ticket');
        const jiraTicket = await jiraClient.getJiraTicket(args.ticketId);
        cli_ux_1.default.action.stop('done');
        return jiraTicket;
    }
    async makePullRequest(args) {
        let githubClient = new github_client_1.GitHubClient(args.accessToken);
        cli_ux_1.default.action.start('Making pull request');
        const result = await githubClient.openPullRequest({
            repo: await nodegit_1.Repository.open('.'),
            title: args.title,
            description: args.description,
            base: args.base,
            draft: args.draft
        });
        cli_ux_1.default.action.stop('done');
        return result;
    }
}
Pr.description = 'create a pull request from a Jira ticket';
Pr.flags = {
    help: command_1.flags.help({ char: 'h' }),
    interactive: command_1.flags.boolean({ char: 'i', description: 'interactive mode', default: false }),
    'base-branch': command_1.flags.string({ char: 'b', description: 'base branch for pull request' }),
    'ticket-id': command_1.flags.string({ char: 't', description: 'Jira ticket ID' }),
    'pr-title': command_1.flags.string({ description: 'custom pull request title' }),
    draft: command_1.flags.boolean({ description: 'make a draft pull request' }),
    description: command_1.flags.string({ description: 'custom pull request description' })
};
exports.default = Pr;
